﻿// JigLibFlashRotationExample Version 1 - Using Papervision 2.0 Beta - Revision 876, JibLibFlash - Revision 53
// A quick demo of newly added Pitch, Roll and Yaw functions I have added to RigidBody class within JigLibFlash. 
// New functions must be added to the RigidBody class for this example to work
//
// released under MIT License
// http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.0.0
//@website 		http://blog.almostlogical.com, http://www.permaven.com


//Copyright (c) 2009 Devin Reimer

/*
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package com
{
	import flash.display.Sprite;
	import flash.events.Event;
	
	import org.papervision3d.cameras.Camera3D;	
	import org.papervision3d.core.math.Number3D;
	import org.papervision3d.view.Viewport3D;
	import org.papervision3d.render.QuadrantRenderEngine;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.materials.shadematerials.FlatShadeMaterial;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.primitives.Sphere;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.scenes.Scene3D;

	import jiglib.math.*;
	import jiglib.geometry.*;
	import jiglib.physics.*;
	import jiglib.physics.constraint.*;

	public class Main extends Sprite
	{	
		private var viewport:Viewport3D;
		private var scene:Scene3D;
		private var camera:Camera3D;
		private var renderer:QuadrantRenderEngine;

		private var lightSource:PointLight3D;
		
		private var jTable:JBox;	
		private var ballArray:Array;
		
		public function Main()
		{	
			graphics.beginFill(0xC9F1ED);
			graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			graphics.endFill();
			
			initPapervision3D();
			createScene();
			beginRender();
		}
		
		private function initPapervision3D():void
		{
			viewport = new Viewport3D(stage.stageWidth,stage.stageHeight);
			addChild(viewport);
			
			scene = new Scene3D();
			 
			lightSource = new PointLight3D(true, true);
			lightSource.x = 100;
			lightSource.y = 300;
			lightSource.z = -500;
					 			
			camera = new Camera3D();	
			camera.y = 450;
			camera.z = -1735;
			camera.rotationX = 15;
			camera.focus = 100;
			camera.zoom = 10;
			 
			renderer = new QuadrantRenderEngine(QuadrantRenderEngine.CORRECT_Z_FILTER);
		}
		
		private function createScene():void
		{
			var tableMatList:MaterialsList = new MaterialsList();
			var tableCubeQuality:int = 8;
			var tableCube:Cube;
			var tableWidth:int = 500;
			var tableDepth:int = 700;
			var tableHeight:int = 100;
			tableMatList.addMaterial(new FlatShadeMaterial(lightSource, 0x6699FF), "front");
			tableMatList.addMaterial(new FlatShadeMaterial(lightSource, 0x6699FF), "back");
			tableMatList.addMaterial(new FlatShadeMaterial(lightSource, 0x0066FF), "left");
			tableMatList.addMaterial(new FlatShadeMaterial(lightSource, 0x0066FF), "right");
			tableMatList.addMaterial(new FlatShadeMaterial(lightSource, 0x0066FF), "top");
			tableMatList.addMaterial(new FlatShadeMaterial(lightSource, 0x0066FF), "bottom");
			tableCube = new Cube(tableMatList,tableWidth, tableDepth, tableHeight, tableCubeQuality, tableCubeQuality, tableCubeQuality);
			
			jTable = new JBox(tableCube,true, tableWidth, tableDepth, tableHeight);
			
			scene.addChild(tableCube);
			PhysicsSystem.getInstance().AddBody(jTable);
			jTable.setMass(20000);
		
			createBalls();
		}
		
		private function createBalls():void
		{
			var ball:JSphere;
			var ballSkin:Sphere;
			var ballRadius:int = 75;
			
			ballArray = new Array();
			
			for (var ballCounter:int = 0; ballCounter < 1; ballCounter++)
			{
				ballSkin = new Sphere(new FlatShadeMaterial(lightSource, 0xFBA248), ballRadius);
				ball = new JSphere(ballSkin, true, ballRadius);
				scene.addChild(ballSkin);
				PhysicsSystem.getInstance().AddBody(ball);
				ball.MoveTo(getRandomBallPosition(), JMatrix3D.IDENTITY);
				ball.setMass(20);
				ballArray.push(ball);
			}
		}
		
		private function getRandomBallPosition():JNumber3D
		{
			return new JNumber3D(Math.random() * 100, 400, Math.random() * 100)
		}
		
		private function beginRender():void
		{
			this.addEventListener(Event.ENTER_FRAME,render);
		}
		private function render(evt:Event):void
		{		
			jTable.roll(((stage.stageWidth / 2) - stage.mouseX) / 500);
			jTable.pitch(((stage.stageHeight / 2) - stage.mouseY) / 350);
			jTable.MoveTo(new JNumber3D(0, 0, 0), jTable.CurrentState.Orientation); //this keeps table from falling (better collision detection on moveable objects (so I'm told)
			
			checkFallenObjects();
			
			PhysicsSystem.getInstance().Integrate(.1);
	
			renderer.renderScene( scene, camera, viewport );
		}
		
		public function checkFallenObjects():void
		{
			for each(var ball:RigidBody in ballArray)
			{
				if (ball.CurrentState.Position.y < -250) {	ball.MoveTo(getRandomBallPosition(), JMatrix3D.IDENTITY);  }
			}
		}
		
	}
}